/*The file contains the function definitions for the event class */

#include <iostream>
using namespace std;
#include "event.h"

//constructor
event::event()
{
  timeout = false;
}

/*********************************************************************
scheduleevent()-Intialize the newly created event node with values for event_time,
                event_type, etc..

	Input: event_time -> time at which the event will occur
               frame_ptr -> pointer to the frame for which the event is associated 
                            with
               event_type -> Type of the event: Timeout, frame handed from node to buffer, 
                             etc.
               see the event.h file for description of other parameters.

	Output: None
***************************************************************************/

void event::scheduleevent(double event_time, frame *frame_ptr, int 
event_type, node *source, node *destn, framebuffer *destbuff, framebuffer 
*sourcebuff, linkx *source_link, linkx *dest_link)
{
   time = event_time;

   switch(event_type)
   {
     case 0:        //event for TO
   	sourcetype = Node;
	desttype = Node;
	sourcenode = source;
        destnode = destn;
        sourcebuffer = sourcebuff;
	destbuffer = destbuff;
	sourcelink = source_link; 
	destlink = dest_link;
	thisframe = NULL;
	dataseq = frame_ptr->dataseq;
     case 1:       //frame handed from node to buffer
	sourcetype = Node;
        desttype = Buffer;
        sourcenode = source;
        destnode = destn;
        sourcebuffer = sourcebuff;
        destbuffer = destbuff;
        sourcelink = source_link;
        destlink = dest_link;
        thisframe = frame_ptr;
        dataseq = frame_ptr->dataseq;
     case 2:		//frame handed from buffer to link
	sourcetype = Buffer;
        desttype = Link;
        sourcenode = source;
        destnode = destn;
        sourcebuffer = sourcebuff;
        destbuffer = destbuff;
        sourcelink = source_link;
        destlink = dest_link;
        thisframe = frame_ptr;
        dataseq = frame_ptr->dataseq;
     case 3:           //frame handed from link to node
	sourcetype = Link;
        desttype = Node;
        sourcenode = source;
        destnode = destn;
        sourcebuffer = sourcebuff;
        destbuffer = destbuff;
        sourcelink = source_link;
        destlink = dest_link;
        thisframe = frame_ptr;
        dataseq = frame_ptr->dataseq;

   };

}

/***********************************************************************
    TO_handler() - Event handler for time out
    Input: head -> Pointer to the event list
           snode -> Pointer to the node in which the event occurred
           PQE_ptr -> Pointer to the PQE node of the event
           current time is the event time
           flink -> link to which the buffer in the source node will give frames to
    Output: None
*************************************************************************/
void event::TO_handler(eventlist *head, node *snode, PQE *PQE_ptr, double 
current_time, linkx *flink)
{
  snode->timeout(dataseq, head, snode, PQE_ptr, current_time, flink); 
  
}

/*********************************************************************
     Node2Buff_handler() - Event handler for event:frame given from node to buffer
     Input: header -> Pointer to the eventlist
            sourcebuff -> Pointer to the buffer to which the frame is given
            current time is the time of the event
     Output: None
**********************************************************************/
void event::Node2Buff_handler(eventlist* header, framebuffer *sourcebuff, 
double current_time)
{
   sourcebuff->include(thisframe, header, sourcebuff, current_time);
    
   // delete thisframe;
    thisframe = NULL;
}


/*********************************************************************
     Buffer2Link_handler() - Event handler for event:frame given from buffer to link
     Input: neweventlist -> Pointer to the event list
            slink -> pointer to the link in which the event occurred
            Current time is the time of the event
     Output: None
***********************************************************************/
void event::Buffer2Link_handler(eventlist * neweventlist, linkx *slink, 
double current_time)
{
  slink->processframe(thisframe, neweventlist, slink, current_time);
  thisframe = NULL;  
}


/************************************************************************
    Link2Node_handler() - Event handler for event:frame given from link to node
    Input: source -> Pointer to the node which sent the frame
           destn -> Pointer to the node to which the frame is given
           header -> Pointer to the eventlist header
           Current time is the time of the event
           flink -> pointer to the link from which the frame is given to the node
    Output: None
***********************************************************************/       
void event::Link2Node_handler(node *source, node *destn,eventlist * 
header, double current_time, linkx *flink)
{
 if(thisframe->type == Data)
   destn->receiveframeatdest(thisframe, destn, header, current_time);
 else
   source->receiveframeatsource(thisframe, source, header, current_time, 
				flink);
  delete thisframe;
  thisframe = NULL;
}

